//==============================================================================
// Cattle Barons
// by Felix Hermansson, Tsunami Studios (hoodncloak@hotmail.com)
// 1 January 2006
//
// Random Map Script for Age of Empires III
// Loosely based on the Great Lakes random map by Ensemble Studios
//==============================================================================
// History:
// 22-12-05: Original release
// 01-01-06: Added missing herd definitions for huntables
//==============================================================================


//==============================================================================
// includes
//==============================================================================

// include mercenaries
include "mercenaries.xs";


//==============================================================================
// function main 
// generates map
//==============================================================================

void main(void)
{
   // Map generation status bar
   rmSetStatusText("",0.01);

   // Define map size
   int playerTiles = 12000;
   if (cNumberNonGaiaPlayers >4)
   {
      playerTiles = 10000;
   }
   int size = 2.0*sqrt(cNumberNonGaiaPlayers*playerTiles);
   rmEchoInfo("Map size = "+size+"m x "+size+"m");
   rmSetMapSize(size, size);  

   // Set lighting
   rmSetLightingSet("Great Plains");

   // Set map elevation parameters
   rmSetMapElevationParameters(cElevTurbulence, 0.02, 7, 0.5, 6.0);
   rmSetMapElevationHeightBlend(1);

   // Pick default terrain		
   rmEnableLocalWater(false);
   rmSetBaseTerrainMix("greatplaines_grass");
   rmTerrainInitialize("great_plaines\ground_grass1_gl", 1.0);
   rmSetMapType("grass");
   rmSetMapType("land");

   // Choose mercs
   chooseMercs();

   // Map generation status bar
   rmSetStatusText("",0.10);


   // Define constraints to have objects and areas avoid each other

   // Define classes needed
   int classPlayer = rmDefineClass("player");
   rmDefineClass("classHill");
   rmDefineClass("classPatch");
   rmDefineClass("starting settlement");
   rmDefineClass("startingUnit");
   rmDefineClass("classForest");
   rmDefineClass("importantItem");
   rmDefineClass("natives");
   rmDefineClass("nuggets");
   rmDefineClass("center");

   // Corner constraint
   rmSetWorldCircleConstraint(true);
   
   // Map edge constraints
   int playerEdgeConstraint = rmCreateBoxConstraint("player edge of map", rmXTilesToFraction(10), rmZTilesToFraction(10), 1.0-rmXTilesToFraction(10), 1.0-rmZTilesToFraction(10), 0.01);
   int longPlayerEdgeConstraint = rmCreateBoxConstraint("long avoid edge of map", rmXTilesToFraction(20), rmZTilesToFraction(20), 1.0-rmXTilesToFraction(20), 1.0-rmZTilesToFraction(20), 0.01);
   
   // Center constraints
   int centerConstraint = rmCreateClassDistanceConstraint("stay away from center", rmClassID("center"), rmXFractionToMeters(0.10));
   int centerConstraintFar = rmCreateClassDistanceConstraint("stay away from center far", rmClassID("center"), rmXFractionToMeters(0.23));
   int centerConstraintExtreme = rmCreateClassDistanceConstraint("stay away from center extremely far", rmClassID("center"), rmXFractionToMeters(0.50));

   // General circle constraint
   int circleConstraint = rmCreatePieConstraint("circle Constraint", 0.5, 0.5, 0, rmZFractionToMeters(0.47), rmDegreesToRadians(0), rmDegreesToRadians(360));

   // Cardinal direction constraints
   int Northward = rmCreatePieConstraint("northMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(315), rmDegreesToRadians(135));
   int Southward = rmCreatePieConstraint("southMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(135), rmDegreesToRadians(315));
   int Eastward = rmCreatePieConstraint("eastMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(45), rmDegreesToRadians(225));
   int Westward = rmCreatePieConstraint("westMapConstraint", 0.5, 0.5, 0, rmZFractionToMeters(0.5), rmDegreesToRadians(225), rmDegreesToRadians(45));

   // Player constraints
   int playerConstraintForest = rmCreateClassDistanceConstraint("forests to stay away from players", classPlayer, 20.0);
   int longPlayerConstraint = rmCreateClassDistanceConstraint("long distance player constraint", classPlayer, 70.0);  
   int mediumPlayerConstraint = rmCreateClassDistanceConstraint("medium distance player constraint", classPlayer, 40.0);  
   int playerConstraint = rmCreateClassDistanceConstraint("standard distance player constraint", classPlayer, 45.0);
   int shortPlayerConstraint = rmCreateClassDistanceConstraint("short distance player constraint", classPlayer, 20.0);

   // Nature constraints (forests, mines & starting resources)
   int forestObjConstraint = rmCreateTypeDistanceConstraint("forest obj", "all", 6.0);
   int forestConstraint = rmCreateClassDistanceConstraint("forest vs. forest", rmClassID("classForest"), 40.0);
   int avoidCoin = rmCreateTypeDistanceConstraint("avoid coin", "Mine", 40.0);
   int shortAvoidCoin = rmCreateTypeDistanceConstraint("short avoid coin", "gold", 10.0);
   int avoidResource = rmCreateTypeDistanceConstraint("resource avoid resource", "resource", 20.0);
   int avoidStartResource = rmCreateTypeDistanceConstraint("start resource no overlap", "resource", 10.0);

   // Impassable land constraints
   int avoidImpassableLand = rmCreateTerrainDistanceConstraint("avoid impassable land", "Land", false, 6.0);
   int shortAvoidImpassableLand = rmCreateTerrainDistanceConstraint("short avoid impassable land", "Land", false, 2.0);
   int longAvoidImpassableLand = rmCreateTerrainDistanceConstraint("long avoid impassable land", "Land", false, 10.0);

   // Unit constraints
   int avoidStartingUnits = rmCreateClassDistanceConstraint("objects avoid starting units", rmClassID("startingUnit"), 45.0);
   int shortAvoidStartingUnits = rmCreateClassDistanceConstraint("objects avoid starting units short", rmClassID("startingUnit"), 10.0);
   int avoidImportantItem = rmCreateClassDistanceConstraint("general items avoid each other", rmClassID("importantItem"), 10.0);
   int avoidNatives = rmCreateClassDistanceConstraint("objects avoid natives", rmClassID("natives"), 30.0);
   int avoidNativesLong = rmCreateClassDistanceConstraint("objects avoid natives long", rmClassID("natives"), 70.0);
   int avoidNuggets = rmCreateClassDistanceConstraint("objects avoid nuggets", rmClassID("nuggets"), 60.0);
   int shortNuggetConstraint = rmCreateTypeDistanceConstraint("avoid nugget objects", "AbstractNugget", 7.0);
   int bisonConstraint = rmCreateTypeDistanceConstraint("bisons avoid bisons", "bison", 40.0);
   int cowConstraint = rmCreateTypeDistanceConstraint("cows avoid cows", "cow", 40.0);

   // General constraint (used mostly for decoration objects)
   int avoidAll = rmCreateTypeDistanceConstraint("avoid all", "all", 6.0);

   // Trade route constraints
   int avoidTradeRoute = rmCreateTradeRouteDistanceConstraint("trade route", 5.0);
   int shortAvoidTradeRoute = rmCreateTradeRouteDistanceConstraint("short trade route", 3.0);
   int avoidTradeRouteFar = rmCreateTradeRouteDistanceConstraint("trade route far", 20.0);
   int avoidTradeSockets = rmCreateTypeDistanceConstraint("avoid trade sockets", "sockettraderoute", 10.0);
   int farAvoidTradeSockets = rmCreateTypeDistanceConstraint("far avoid trade sockets", "sockettraderoute", 40.0);

   // Map generation status bar
   rmSetStatusText("",0.20);


   // Create trade route

   // Define sockets
   int socketID = rmCreateObjectDef("sockets to dock Trade Posts");
   rmAddObjectDefItem(socketID, "SocketTradeRoute", 1, 0.0);
   rmSetObjectDefAllowOverlap(socketID, true);
   rmSetObjectDefMinDistance(socketID, 0.0);
   rmSetObjectDefMaxDistance(socketID, 6.0);
   rmAddObjectDefConstraint(socketID, shortAvoidTradeRoute);

   // Create dirt route
   int tradeRouteID = rmCreateTradeRoute();
   rmSetObjectDefTradeRouteID(socketID, tradeRouteID);
   rmAddTradeRouteWaypoint(tradeRouteID, 0.08, 0.55);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.20, 0.83, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.45, 0.93, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.67, 0.89, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.78, 0.78, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.84, 0.70, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.86, 0.68, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.88, 0.40, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.81, 0.23, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.70, 0.10, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.50, 0.08, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.30, 0.15, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.15, 0.23, 2, 5);
   rmAddRandomTradeRouteWaypoints(tradeRouteID, 0.08, 0.55, 2, 5);
   bool placedTradeRoute = rmBuildTradeRoute(tradeRouteID, "dirt");
   if(placedTradeRoute == false)
   {
      rmEchoError("Failed to place trade route");
   }

   // Place sockets at fixed positions
   rmPlaceObjectDefAtLoc(socketID, 0, 0.24, 0.81);
   rmPlaceObjectDefAtLoc(socketID, 0, 0.59, 0.88); 
   rmPlaceObjectDefAtLoc(socketID, 0, 0.17, 0.21);
   rmPlaceObjectDefAtLoc(socketID, 0, 0.86, 0.68);
   rmPlaceObjectDefAtLoc(socketID, 0, 0.85, 0.23); 
   rmPlaceObjectDefAtLoc(socketID, 0, 0.45, 0.12);

   // Map generation status bar
   rmSetStatusText("",0.30);


   // Place players

   // Get number of players per team
   int teamZeroCount = rmGetNumberPlayersOnTeam(0);
   int teamOneCount = rmGetNumberPlayersOnTeam(1);

   // Place players either in arcs per team or in a circle
   if ( cNumberTeams <= 2 && teamZeroCount <= 4 && teamOneCount <= 4)
   {
      rmSetPlacementTeam(0);
      rmSetPlacementSection(0.65, 0.85);
      rmSetTeamSpacingModifier(0.25);
      rmPlacePlayersCircular(0.30, 0.32, 0);
      rmSetPlacementTeam(1);
      rmSetPlacementSection(0.15, 0.35);
      rmSetTeamSpacingModifier(0.25);
      rmPlacePlayersCircular(0.30, 0.32, 0);
   }
   else
   {
      rmSetTeamSpacingModifier(0.7);
      rmPlacePlayersCircular(0.32, 0.34, 0.0);
   }

   // Create player areas
   float playerFraction = rmAreaTilesToFraction(100);
   for(i=1; <cNumberPlayers)
   {
      int id = rmCreateArea("Player"+i);
      rmSetPlayerArea(i, id);
      rmSetAreaSize(id, playerFraction, playerFraction);
      rmAddAreaToClass(id, classPlayer);
      rmSetAreaMinBlobs(id, 1);
      rmSetAreaMaxBlobs(id, 1);
      rmAddAreaConstraint(id, playerConstraint); 
      rmAddAreaConstraint(id, playerEdgeConstraint); 
      rmAddAreaConstraint(id, longAvoidImpassableLand);
      rmSetAreaLocPlayer(id, i);
      rmSetAreaWarnFailure(id, false);
   }

   // Build all areas
   rmBuildAllAreas();

   // Map generation status bar
   rmSetStatusText("",0.40);


   // Define and place starting units and resources

   // Define TC (ignoring nomad start)
   int startingTCID =  rmCreateObjectDef("startingTC");
   if (rmGetNomadStart())
   {
      // rmAddObjectDefItem(startingTCID, "CoveredWagon", 1, 5.0);
      rmAddObjectDefItem(startingTCID, "townCenter", 1, 5.0); // ignore nomad start
   }
   else
   {
      rmAddObjectDefItem(startingTCID, "townCenter", 1, 5.0);
   }
   rmAddObjectDefConstraint(startingTCID, avoidImpassableLand);
   rmAddObjectDefConstraint(startingTCID, avoidTradeRoute);
   rmAddObjectDefToClass(startingTCID, rmClassID("player"));

   // Define starting units (explorer, envoy, native scout & war dog
   int startingUnits = rmCreateStartingUnitsObjectDef(5.0);
   rmSetObjectDefMinDistance(startingUnits, 10.0);
   rmSetObjectDefMaxDistance(startingUnits, 12.0);
   rmAddObjectDefToClass(startingUnits, rmClassID("startingUnit"));

   // Define starting pen
   int startPenID = rmCreateObjectDef("livestock pen");
   rmAddObjectDefItem(startPenID, "LivestockPen", 1, 0.0);
   rmAddObjectDefConstraint(startPenID, avoidTradeRoute);
   rmSetObjectDefMinDistance(startPenID, 15.0);
   rmSetObjectDefMaxDistance(startPenID, 15.0);
   rmAddObjectDefConstraint(startPenID, avoidStartResource);
   rmAddObjectDefConstraint(startPenID, avoidImpassableLand);

   // Define starting mine
   int startSilverID = rmCreateObjectDef("player silver");
   rmAddObjectDefItem(startSilverID, "mine", 1, 5.0);
   rmAddObjectDefConstraint(startSilverID, shortAvoidTradeRoute);
   rmSetObjectDefMinDistance(startSilverID, 12.0);
   rmSetObjectDefMaxDistance(startSilverID, 20.0);
   rmAddObjectDefConstraint(startSilverID, avoidStartResource);
   rmAddObjectDefConstraint(startSilverID, avoidImpassableLand);

   // Define starting cows
   int startCowID = rmCreateObjectDef("starting cows");
   rmAddObjectDefItem(startCowID, "cow", 3, 4.0);
   rmSetObjectDefMinDistance(startCowID, 10);
   rmSetObjectDefMaxDistance(startCowID, 16);
   rmAddObjectDefConstraint(startCowID, avoidStartResource);
   rmAddObjectDefConstraint(startCowID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(startCowID, avoidNatives);

   // Define starting bisons
   int startBisonID = rmCreateObjectDef("starting bisons");
   rmAddObjectDefItem(startBisonID, "bison", 2, 6.0);
   rmSetObjectDefMinDistance(startBisonID, 16);
   rmSetObjectDefMaxDistance(startBisonID, 20);
   rmAddObjectDefConstraint(startBisonID, avoidStartResource);
   rmAddObjectDefConstraint(startBisonID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(startBisonID, avoidNatives);
   rmSetObjectDefCreateHerd(startBisonID, true);

   // Define starting trees
   int startAreaTreeID = rmCreateObjectDef("starting trees");
   rmAddObjectDefItem(startAreaTreeID, "TreeGreatPlains", 4, 6.0);
   rmSetObjectDefMinDistance(startAreaTreeID, 6);
   rmSetObjectDefMaxDistance(startAreaTreeID, 16);
   rmAddObjectDefConstraint(startAreaTreeID, avoidStartResource);
   rmAddObjectDefConstraint(startAreaTreeID, shortAvoidImpassableLand);
   rmAddObjectDefConstraint(startAreaTreeID, avoidTradeRoute);
   rmAddObjectDefConstraint(startAreaTreeID, avoidNatives);
   rmAddObjectDefConstraint(startAreaTreeID, shortAvoidStartingUnits);

   // Place starting units and resources for all payers 
   for(i=1; <cNumberPlayers)
   {
      rmPlaceObjectDefAtLoc(startingTCID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));      
      rmPlaceObjectDefAtLoc(startSilverID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startingUnits, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startBisonID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startCowID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startCowID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startPenID, i, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
      rmPlaceObjectDefAtLoc(startAreaTreeID, 0, rmPlayerLocXFraction(i), rmPlayerLocZFraction(i));
   }

   // Map generation status bar
   rmSetStatusText("",0.50);


   // Define and place natives

   // Choose which natives (Lakota or Comanche) appear on the map
   int subCiv0 = -1;
   int subCiv1 = -1;
   float nativeChooser = rmRandFloat(0,1);

   if (rmAllocateSubCivs(2) == true)
   {
      if (nativeChooser <= 0.5)
      {
         subCiv0 = rmGetCivID("Lakota");
         if (subCiv0 >= 0)
         {
            rmSetSubCiv(0, "Lakota", true);
         }
         subCiv1 = rmGetCivID("Lakota");
         if (subCiv1 >= 0)
         {
            rmSetSubCiv(1, "Lakota", true);
         }
      }
      else
      {
         subCiv0 = rmGetCivID("Comanche");
         if (subCiv0 >= 0)
         {
            rmSetSubCiv(0, "Comanche", true);
         }
         subCiv1 = rmGetCivID("Comanche");
         if (subCiv1 >= 0)
         {
            rmSetSubCiv(1, "Comanche", true);
         }
      }
   }

   // Define area for native villages
   int nativeAreaId = rmCreateArea("native area");
   rmSetAreaSize(nativeAreaId, rmAreaTilesToFraction(2000), rmAreaTilesToFraction(2000));
   rmSetAreaLocation(nativeAreaId, 0.5, 0.5);
   rmSetAreaCoherence(nativeAreaId, 0.3);
   rmSetAreaWarnFailure(nativeAreaId, false);
   rmBuildArea(nativeAreaId);

   // Define and place native villages
   int nativeVillage1ID = -1;
   int nativeVillage2ID = -1;
   if (subCiv0 == rmGetCivID("Lakota"))
   {
      // Lakota village 1
      int lakotaVillage1Type = rmRandInt(1,5);
      nativeVillage1ID = rmCreateGrouping("native village 1", "native lakota village "+lakotaVillage1Type);
      rmSetGroupingMinDistance(nativeVillage1ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage1ID, 30.0);
      rmAddGroupingConstraint(nativeVillage1ID, longPlayerConstraint);
      rmAddGroupingConstraint(nativeVillage1ID, avoidNativesLong);
      rmAddGroupingConstraint(nativeVillage1ID, farAvoidTradeSockets);
      rmAddGroupingToClass(nativeVillage1ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage1ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage1ID, 0, nativeAreaId);

      // Lakota village 2
      int lakotaVillage2Type = rmRandInt(1,5);
      nativeVillage2ID = rmCreateGrouping("native village 2", "native lakota village "+lakotaVillage2Type);
      rmSetGroupingMinDistance(nativeVillage2ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage2ID, 30.0);
      rmAddGroupingConstraint(nativeVillage2ID, longPlayerConstraint);
      rmAddGroupingConstraint(nativeVillage2ID, avoidNativesLong);
      rmAddGroupingConstraint(nativeVillage2ID, farAvoidTradeSockets);
      rmAddGroupingToClass(nativeVillage2ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage2ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage2ID, 0, nativeAreaId);

   }
   else
   {
      // Comanche village 1
      int comancheVillage1Type = rmRandInt(1,5);
      nativeVillage1ID = rmCreateGrouping("native village 1", "native comanche village "+comancheVillage1Type);
      rmSetGroupingMinDistance(nativeVillage1ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage1ID, 30.0);
      rmAddGroupingConstraint(nativeVillage1ID, longPlayerConstraint);
      rmAddGroupingConstraint(nativeVillage1ID, avoidNativesLong);
      rmAddGroupingConstraint(nativeVillage1ID, farAvoidTradeSockets);
      rmAddGroupingToClass(nativeVillage1ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage1ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage1ID, 0, nativeAreaId);

      // Comanche village 2
      int comancheVillage2Type = rmRandInt(1,5);
      nativeVillage2ID = rmCreateGrouping("native village 2", "native comanche village "+comancheVillage2Type);
      rmSetGroupingMinDistance(nativeVillage2ID, 0.0);
      rmSetGroupingMaxDistance(nativeVillage2ID, 30.0);
      rmAddGroupingConstraint(nativeVillage2ID, longPlayerConstraint);
      rmAddGroupingConstraint(nativeVillage2ID, avoidNativesLong);
      rmAddGroupingConstraint(nativeVillage2ID, farAvoidTradeSockets);
      rmAddGroupingToClass(nativeVillage2ID, rmClassID("natives"));
      rmAddGroupingToClass(nativeVillage2ID, rmClassID("importantItem"));
      rmPlaceGroupingInArea(nativeVillage2ID, 0, nativeAreaId);
   }

   // Map generation status bar
   rmSetStatusText("",0.60);


   // Place forests and silver mines

   // Place silver mines
   int silverCount = (cNumberNonGaiaPlayers * 2) + 1;
   int silverMineID = rmCreateObjectDef("silver mines");
   rmAddObjectDefItem(silverMineID, "mine", 1, 0.0);
   rmSetObjectDefMinDistance(silverMineID, 0.0);
   rmSetObjectDefMaxDistance(silverMineID, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(silverMineID, avoidImpassableLand);
   rmAddObjectDefConstraint(silverMineID, avoidCoin);
   rmAddObjectDefConstraint(silverMineID, longPlayerConstraint);
   rmAddObjectDefConstraint(silverMineID, avoidAll);
   rmAddObjectDefConstraint(silverMineID, avoidTradeRoute);
   rmAddObjectDefConstraint(silverMineID, avoidTradeSockets);
   for(i=0; <silverCount)
   { 
      rmPlaceObjectDefAtLoc(silverMineID, 0, 0.5, 0.5);
   }

   // Place forests
   int numTries = -1;
   int failCount = -1;
   int forestTreeID = 0;
   numTries=5*cNumberNonGaiaPlayers;
   failCount=0;
   for (i=0; <numTries)  // Northern forests
   {   
      int northForest=rmCreateArea("northforest"+i);
      rmSetAreaWarnFailure(northForest, false);
      rmSetAreaSize(northForest, rmAreaTilesToFraction(20), rmAreaTilesToFraction(100));
      rmSetAreaForestType(northForest, "great plains forest");
      rmSetAreaForestDensity(northForest, 0.4);
      rmAddAreaToClass(northForest, rmClassID("classForest"));
      rmSetAreaForestClumpiness(northForest, 0.0);
      rmSetAreaForestUnderbrush(northForest, 0.0);
      rmSetAreaBaseHeight(northForest, 2.0);
      rmSetAreaMinBlobs(northForest, 1);
      rmSetAreaMaxBlobs(northForest, 3);
      rmSetAreaMinBlobDistance(northForest, 16.0);
      rmSetAreaMaxBlobDistance(northForest, 30.0);
      rmSetAreaCoherence(northForest, 0.7);
      rmSetAreaSmoothDistance(northForest, 1);
      rmAddAreaConstraint(northForest, avoidImportantItem);
      rmAddAreaConstraint(northForest, avoidTradeRoute);
      rmAddAreaConstraint(northForest, avoidTradeSockets);
      rmAddAreaConstraint(northForest, shortAvoidCoin);
      rmAddAreaConstraint(northForest, playerConstraintForest);
      rmAddAreaConstraint(northForest, forestConstraint);
      rmAddAreaConstraint(northForest, Northward);
      if(rmBuildArea(northForest)==false)
      {
         // Stop trying once we fail 5 times in a row.  
         failCount++;
         if(failCount == 5)
         {      
		break;
         }
      }
      else
      {
         failCount=0;
      }
   }
   failCount=0;
   for (i=0; <numTries)  // Southern forests
   {   
      int southForest=rmCreateArea("southforest"+i);
      rmSetAreaWarnFailure(southForest, false);
      rmSetAreaSize(southForest, rmAreaTilesToFraction(20), rmAreaTilesToFraction(100));
      rmSetAreaForestType(southForest, "great plains forest");
      rmSetAreaForestDensity(southForest, 0.4);
      rmAddAreaToClass(southForest, rmClassID("classForest"));
      rmSetAreaForestClumpiness(southForest, 0.0);
      rmSetAreaForestUnderbrush(southForest, 0.0);
      rmSetAreaBaseHeight(southForest, 2.0);
      rmSetAreaMinBlobs(southForest, 1);
      rmSetAreaMaxBlobs(southForest, 3);
      rmSetAreaMinBlobDistance(southForest, 16.0);
      rmSetAreaMaxBlobDistance(southForest, 30.0);
      rmSetAreaCoherence(southForest, 0.7);
      rmSetAreaSmoothDistance(southForest, 1);
      rmAddAreaConstraint(southForest, avoidImportantItem);
      rmAddAreaConstraint(southForest, avoidTradeRoute);
      rmAddAreaConstraint(southForest, avoidTradeSockets);
      rmAddAreaConstraint(southForest, shortAvoidCoin);
      rmAddAreaConstraint(southForest, playerConstraintForest);
      rmAddAreaConstraint(southForest, forestConstraint);
      rmAddAreaConstraint(southForest, Southward);
      if(rmBuildArea(southForest)==false)
      {
         // Stop trying once we fail 5 times in a row.  
         failCount++;
         if(failCount == 5)
         {      
		break;
         }
      }
      else
      {
         failCount=0;
      }
   }

   // Map generation status bar
   rmSetStatusText("",0.70);


   // Place animals

   // Place bison herds
   int bisonCount = cNumberNonGaiaPlayers + rmRandInt(1,cNumberNonGaiaPlayers);
   int bisonHerdID = rmCreateObjectDef("bison herd");
   rmAddObjectDefItem(bisonHerdID, "bison", rmRandInt(3,4), 6.0);
   rmSetObjectDefMinDistance(bisonHerdID, 0.0);
   rmSetObjectDefMaxDistance(bisonHerdID, rmXFractionToMeters(0.25));
   rmAddObjectDefConstraint(bisonHerdID, avoidImpassableLand);
   rmAddObjectDefConstraint(bisonHerdID, avoidCoin);
   rmAddObjectDefConstraint(bisonHerdID, longPlayerConstraint);
   rmAddObjectDefConstraint(bisonHerdID, bisonConstraint);
   rmAddObjectDefConstraint(bisonHerdID, avoidTradeRoute);
   rmSetObjectDefCreateHerd(bisonHerdID, true);
   for(i=0; <bisonCount)
   { 
      rmPlaceObjectDefAtLoc(bisonHerdID, 0, 0.5, 0.5);
   }
   // Place additional cows
   int cowCount = (2*cNumberNonGaiaPlayers) + rmRandInt(0,cNumberNonGaiaPlayers) + rmRandInt(0,cNumberNonGaiaPlayers);
   int cowHerd1ID = rmCreateObjectDef("cow herd north");
   rmAddObjectDefItem(cowHerd1ID, "cow", 3, 8.0);
   rmSetObjectDefMinDistance(cowHerd1ID, 0.20);
   rmSetObjectDefMaxDistance(cowHerd1ID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(cowHerd1ID, avoidImpassableLand);
   rmAddObjectDefConstraint(cowHerd1ID, shortAvoidCoin);
   rmAddObjectDefConstraint(cowHerd1ID, playerConstraint);
   rmAddObjectDefConstraint(cowHerd1ID, cowConstraint);
   rmAddObjectDefConstraint(cowHerd1ID, Northward);
   for(i=0; <cowCount)
   { 
      rmPlaceObjectDefAtLoc(cowHerd1ID, 0, 0.5, 0.5); // northern half of the map
   }
   int cowHerd2ID = rmCreateObjectDef("cow herd south");
   rmAddObjectDefItem(cowHerd2ID, "cow", 3, 8.0);
   rmSetObjectDefMinDistance(cowHerd2ID, 0.20);
   rmSetObjectDefMaxDistance(cowHerd2ID, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(cowHerd2ID, avoidImpassableLand);
   rmAddObjectDefConstraint(cowHerd2ID, shortAvoidCoin);
   rmAddObjectDefConstraint(cowHerd2ID, playerConstraint);
   rmAddObjectDefConstraint(cowHerd2ID, cowConstraint);
   rmAddObjectDefConstraint(cowHerd2ID, Southward);
   for(i=0; <cowCount)
   { 
      rmPlaceObjectDefAtLoc(cowHerd2ID, 0, 0.5, 0.5); // southern half of the map
   }

   // Map generation status bar
   rmSetStatusText("",0.80);


  // Define and place Nuggets

  // Easy nuggets (2 per player, close to the TC)
   int nugget1= rmCreateObjectDef("nugget easy"); 
   rmAddObjectDefItem(nugget1, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(1, 1);
   rmAddObjectDefToClass(nugget1, rmClassID("nuggets"));
   rmAddObjectDefConstraint(nugget1, shortPlayerConstraint);
   rmAddObjectDefConstraint(nugget1, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget1, avoidNuggets);
   rmAddObjectDefConstraint(nugget1, avoidTradeSockets);
   rmAddObjectDefConstraint(nugget1, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget1, avoidAll);
   rmAddObjectDefConstraint(nugget1, circleConstraint);
   rmSetObjectDefMinDistance(nugget1, 30.0);
   rmSetObjectDefMaxDistance(nugget1, 50.0);
   rmPlaceObjectDefPerPlayer(nugget1, false, 2);

   // Medium nuggets (1 per player, further from the TC)
   int nugget2= rmCreateObjectDef("nugget medium"); 
   rmAddObjectDefItem(nugget2, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(2, 2);
   rmAddObjectDefToClass(nugget2, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(nugget2, 0.0);
   rmSetObjectDefMaxDistance(nugget2, rmXFractionToMeters(0.5));
   rmAddObjectDefConstraint(nugget2, shortPlayerConstraint);
   rmAddObjectDefConstraint(nugget2, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget2, avoidNuggets);
   rmAddObjectDefConstraint(nugget2, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget2, circleConstraint);
   rmAddObjectDefConstraint(nugget2, avoidAll);
   rmSetObjectDefMinDistance(nugget2, 50.0);
   rmSetObjectDefMaxDistance(nugget2, 100.0);
   rmPlaceObjectDefPerPlayer(nugget2, false, 1);

   // Hard nuggets (1 per player, central part of the map)
   int nugget3= rmCreateObjectDef("nugget hard"); 
   rmAddObjectDefItem(nugget3, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(3, 3);
   rmAddObjectDefToClass(nugget3, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(nugget3, 0.15);
   rmSetObjectDefMaxDistance(nugget3, rmXFractionToMeters(0.35));
   rmAddObjectDefConstraint(nugget3, shortPlayerConstraint);
   rmAddObjectDefConstraint(nugget3, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget3, avoidNuggets);
   rmAddObjectDefConstraint(nugget3, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget3, circleConstraint);
   rmAddObjectDefConstraint(nugget3, avoidAll);
   rmPlaceObjectDefAtLoc(nugget3, 0, 0.5, 0.5, cNumberNonGaiaPlayers);

   // Very hard nuggets (2 to 6 in total, central part of the map)
   int nugget4= rmCreateObjectDef("nugget nuts"); 
   rmAddObjectDefItem(nugget4, "Nugget", 1, 0.0);
   rmSetNuggetDifficulty(4, 4);
   rmAddObjectDefToClass(nugget4, rmClassID("nuggets"));
   rmSetObjectDefMinDistance(nugget4, 0.15);
   rmSetObjectDefMaxDistance(nugget4, rmXFractionToMeters(0.25));
   rmAddObjectDefConstraint(nugget4, longPlayerConstraint);
   rmAddObjectDefConstraint(nugget4, avoidImpassableLand);
   rmAddObjectDefConstraint(nugget4, avoidNuggets);
   rmAddObjectDefConstraint(nugget4, avoidTradeRoute);
   rmAddObjectDefConstraint(nugget4, circleConstraint);
   rmAddObjectDefConstraint(nugget4, avoidAll);
   rmPlaceObjectDefAtLoc(nugget4, 0, 0.5, 0.5, rmRandInt(2,6));

   // Map generation status bar
   rmSetStatusText("",0.90);


   // Add decoration

   // Place single trees
   int loneTree = rmCreateObjectDef("lone trees");
   rmAddObjectDefItem(loneTree, "TreeGreatPlains", 1, 0.0);
   rmSetObjectDefMinDistance(loneTree, 0.0);
   rmSetObjectDefMaxDistance(loneTree, rmXFractionToMeters(0.50));
   rmAddObjectDefConstraint(loneTree, shortPlayerConstraint);
   rmAddObjectDefConstraint(loneTree, avoidTradeRoute);
   rmAddObjectDefConstraint(loneTree, avoidAll);
   rmPlaceObjectDefAtLoc(loneTree, 0, 0.5, 0.5, size/15);

   // Place some carcasses
   int carcassCount = rmRandInt(1,3);
   int bisonCarcass = rmCreateGrouping("bison carcass", "gp_carcass_bison");
   rmSetGroupingMinDistance(bisonCarcass, 0.0);
   rmSetGroupingMaxDistance(bisonCarcass, rmXFractionToMeters(0.50));
   rmAddGroupingConstraint(bisonCarcass, playerConstraint);
   rmAddGroupingConstraint(bisonCarcass, avoidNatives);
   rmAddGroupingConstraint(bisonCarcass, avoidTradeRoute);
   rmAddGroupingConstraint(bisonCarcass, avoidAll);
   rmPlaceGroupingAtLoc(bisonCarcass, 0, 0.5, 0.5, carcassCount);

   // Place grass patches
   int grassPatchGroupType = rmRandInt(1,7);
   int grassPatchGroup = rmCreateGrouping("grass patch group"+i, "gp_grasspatch0"+grassPatchGroupType);
   rmSetGroupingMinDistance(grassPatchGroup, 0.0);
   rmSetGroupingMaxDistance(grassPatchGroup, rmXFractionToMeters(0.50));
   rmAddGroupingConstraint(grassPatchGroup, playerConstraint);
   rmAddGroupingConstraint(grassPatchGroup, avoidNatives);
   rmAddGroupingConstraint(grassPatchGroup, avoidTradeRoute);
   rmAddGroupingConstraint(grassPatchGroup, avoidAll);
   rmPlaceGroupingAtLoc(grassPatchGroup, 0, 0.5, 0.5, size/50);

   // Place flower patches
   int flowerPatchGroupType = rmRandInt(1,8);
   int flowerPatchGroup = rmCreateGrouping("flower patch group"+i, "gp_flower0"+flowerPatchGroupType);
   rmSetGroupingMinDistance(flowerPatchGroup, 0.0);
   rmSetGroupingMaxDistance(flowerPatchGroup, rmXFractionToMeters(0.50));
   rmAddGroupingConstraint(flowerPatchGroup, playerConstraint);
   rmAddGroupingConstraint(flowerPatchGroup, avoidNatives);
   rmAddGroupingConstraint(flowerPatchGroup, avoidTradeRoute);
   rmAddGroupingConstraint(flowerPatchGroup, avoidAll);
   rmPlaceGroupingAtLoc(flowerPatchGroup, 0, 0.5, 0.5, size/100);

   // Map generation status bar
   rmSetStatusText("",1.0);

}  
